/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.process.internal.definitions.ide.ui;

import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;

import org.eclipse.osgi.util.NLS;

import org.eclipse.swt.graphics.Image;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreePathContentProvider;
import org.eclipse.jface.viewers.ITreePathLabelProvider;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerLabel;

import org.eclipse.ui.IWorkbenchPartSite;

import com.ibm.team.repository.common.IFeedServiceConstants;

import com.ibm.team.repository.client.ITeamRepository;

import com.ibm.team.feed.core.FeedManager;
import com.ibm.team.feed.core.model.Channel;
import com.ibm.team.feed.core.model.FeedFactory;
import com.ibm.team.feed.ui.internal.OpenFeedAction;
import com.ibm.team.foundation.common.IFeedConstants;
import com.ibm.team.jface.labelProviders.TypeLabelProvider;

import com.ibm.team.process.common.ITeamArea;

import com.ibm.team.process.rcp.ui.teamnavigator.Category;
import com.ibm.team.process.rcp.ui.teamnavigator.Domain;
import com.ibm.team.process.rcp.ui.teamnavigator.DomainSubtreeRoot;
import com.ibm.team.process.rcp.ui.teamnavigator.IProcessAreaCategory;

public class FeedDomain extends Domain {
	
	
	private static class FeedLabelProvider extends TypeLabelProvider implements ITreePathLabelProvider {
		
		public FeedLabelProvider() {
		}
		
		/*
		 * @see com.ibm.team.jface.labelProviders.TypeLabelProvider#updateLabel(org.eclipse.jface.viewers.ViewerLabel, java.lang.Object)
		 */
		public void updateLabel(ViewerLabel label, Object element) {
			if (element instanceof Channel) {
				ImageDescriptor descriptor = DefinitionsIdeUIPlugin.getImageDescriptor("icons/obj16/feeds_evnt_etern_obj.gif"); //$NON-NLS-1$
				Image image = getResources().createImageWithDefault(descriptor);
				label.setImage(image);
				
				Channel channel = (Channel) element;
				label.setText(channel.getTitle());
			}
		}
		
		/*
		 * @see org.eclipse.jface.viewers.ITreePathLabelProvider#updateLabel(org.eclipse.jface.viewers.ViewerLabel, org.eclipse.jface.viewers.TreePath)
		 */
		public void updateLabel(ViewerLabel label, TreePath elementPath) {
			updateLabel(label, elementPath.getLastSegment());
		}
	}
	
	
	private class FeedContentProvider implements ITreePathContentProvider {
		/*
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}

		/*
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {
		}
	
		/*
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			// unused method
			return new Object[0];
		}
	
		/*
		 * @see org.eclipse.jface.viewers.ITreePathContentProvider#hasChildren(org.eclipse.jface.viewers.TreePath)
		 */
		public boolean hasChildren(TreePath path) {
			return (path.getLastSegment() instanceof DomainSubtreeRoot);
		}
	
		/*
		 * @see org.eclipse.jface.viewers.ITreePathContentProvider#getParents(java.lang.Object)
		 */
		public TreePath[] getParents(Object element) {
			return new TreePath[0];
		}
	
		/*
		 * @see org.eclipse.jface.viewers.ITreePathContentProvider#getChildren(org.eclipse.jface.viewers.TreePath)
		 */
		public Object[] getChildren(TreePath parentPath) {
			if (parentPath.getLastSegment() instanceof DomainSubtreeRoot) {
				ITeamArea area = (ITeamArea) ((DomainSubtreeRoot) parentPath.getLastSegment()).getCategoryElement();
				Channel channel = createChannel(area);
				if (channel != null) {
					return new Object[] { channel };
				}
			}
			return new Object[0];
		}
	}

	public FeedDomain() {
		fFeedLabelProvider = new FeedLabelProvider();
	}
	
	/*
	 * @see com.ibm.team.process.rcp.ui.teamnavigator.Domain#supportsCategory(com.ibm.team.process.rcp.ui.teamnavigator.Category)
	 */
	public boolean supportsCategory(Category category) {
		if (category instanceof IProcessAreaCategory) {
			return ((IProcessAreaCategory) category).isTeamAreaCategory();
		}
		return false;
	}
	
	/*
	 * @see com.ibm.team.process.rcp.ui.teamnavigator.Domain#confineToSupportedCategories()
	 */
	public boolean confineToSupportedCategories() {
		return true;
	}

	/*
	 * @see com.ibm.team.process.rcp.ui.teamnavigator.Domain#contains(java.lang.Object)
	 */
	public boolean contains(Object element) {
		return (element instanceof Channel);
	}

	/*
	 * @see com.ibm.team.process.rcp.ui.teamnavigator.Domain#getContentProvider()
	 */
	public ITreePathContentProvider getContentProvider() {
		return new FeedContentProvider();
	}

	/*
	 * @see com.ibm.team.process.rcp.ui.teamnavigator.Domain#getLabelProvider()
	 */
	public ITreePathLabelProvider getLabelProvider() {
		return fFeedLabelProvider;
	}
		
	/*
	 * @see com.ibm.team.process.rcp.ui.teamnavigator.Domain#open(org.eclipse.ui.IWorkbenchPartSite, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public boolean open(IWorkbenchPartSite site, IStructuredSelection selection) {
		if (selection.getFirstElement() == getRoot())
			return false;

		OpenFeedAction action= new OpenFeedAction(selection, site.getPage());
		action.run(null);
		return true;
	}
	
	
	// --------------- helpers -------------------------------------------------
	
	private static final String DFLT_ENCODING= "UTF-8"; //$NON-NLS-1$
	private static final String FEED_SERVICE= IFeedServiceConstants.SERVICE_NAME;
	private FeedLabelProvider fFeedLabelProvider;

	private Channel createChannel(ITeamArea teamArea) {

		
		String url= null;
		try {
			ITeamRepository teamRepository = (ITeamRepository) teamArea.getOrigin();
			url= buildURL(teamRepository, teamArea);
		} catch (Exception e) {
			DefinitionsIdeUIPlugin.getDefault().log(e);
			return null;
		}
		
		Channel channel= FeedFactory.eINSTANCE.createChannel();
		channel.setUrl(url);
		channel.setTitle(Messages.FeedDomain_2);
		channel.setDescription(NLS.bind(Messages.FeedDomain_3, teamArea.getName()));
		channel.setForceDownload(true);
		channel.setCategory(IFeedConstants.SYSTEM_FEED_CATEGORY);
//		channel.setUpdateInterval(10);
//		channel.setColor(SWT.COLOR_DARK_MAGENTA);
		
		FeedManager.getDefault().loadNews(channel);
		
		return channel;
	}

	private String buildURL(ITeamRepository repo, ITeamArea teamArea) throws UnsupportedEncodingException, URISyntaxException {
		StringBuffer url= new StringBuffer();
		url.append(getBaseURI(new URI(repo.getRepositoryURI())));
		url.append("service/"); //$NON-NLS-1$
		url.append(FEED_SERVICE);
		url.append("?provider=team&team_area="); //$NON-NLS-1$
		url.append(teamArea.getItemId().getUuidValue());
		url.append("&user="); //$NON-NLS-1$
		url.append(URLEncoder.encode(repo.loggedInContributor().getUserId(), DFLT_ENCODING));
		return url.toString();
	}
	
	public static String getBaseURI(URI uri) {
		
		String protocol= uri.getScheme();
		String authority= uri.getAuthority();
		String path= uri.getPath();
		
		if (protocol == null || authority == null || path == null)
			return null;
		
		// shrink path to application root
		String[] segments= path.split("/"); // null check above //$NON-NLS-1$
		int i= 0;
		while (i < segments.length && segments[i].equals("")) //$NON-NLS-1$
			i++;
		path= "/"; //$NON-NLS-1$
		if (i < segments.length)
			path+= segments[i]+"/"; //$NON-NLS-1$
		
		// reconstruct base URI from 
		StringBuffer url= new StringBuffer();
		url.append(protocol);
		url.append("://"); //$NON-NLS-1$
		url.append(authority);
		url.append(path);
		return url.toString();
	}
	
	/*
	 * @see com.ibm.team.process.rcp.ui.teamnavigator.Domain#dispose()
	 */
	public void dispose() {
		fFeedLabelProvider.dispose();
		super.dispose();
	}
}