/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.process.internal.definitions.ide.ui.deliver.editors;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.osgi.util.NLS;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Tree;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.layout.LayoutConstants;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.jface.window.Window;

import org.eclipse.ui.forms.widgets.FormToolkit;

import com.ibm.team.foundation.common.util.IMemento;

import com.ibm.team.process.ide.ui.IProcessAspectEditorSite;
import com.ibm.team.process.ide.ui.OperationDetailsAspectEditor;
import com.ibm.team.process.ide.ui.ProcessAspect;

import com.ibm.team.process.internal.ide.ui.ImagePool;
import com.ibm.team.process.internal.ide.ui.editors.form.AbstractProcessAspect;
import com.ibm.team.process.internal.ide.ui.editors.form.IDevelopmentLineProxy;
import com.ibm.team.process.internal.ide.ui.editors.form.IIterationStructureProxy;

/**
 * Aspect editor for the defining a team area hierarchy to be created in project area initialization.
 */
public class CreateTeamAreasAspectEditor extends OperationDetailsAspectEditor {
	
//	<teamArea name="t1" timeline="l1">
//		<teamArea name="t11">
//			<teamArea name="t111"></teamArea>
//			<teamArea name="t112"></teamArea>
//		</teamArea>
//	</teamArea>
//	<teamArea name="t2" timeline="l2">
//		<teamArea name="t21"></teamArea>
//	</teamArea>

	private static final String TAG_TEAM_AREA = "teamArea"; //$NON-NLS-1$
	private static final String ATTR_NAME = "name"; //$NON-NLS-1$
	private static final String ATTR_DEVELOPMENT_LINE = "timeline"; //$NON-NLS-1$
	

	private static class Model {
		
		static final String SEPARATOR = "/"; //$NON-NLS-1$
		
		final IDevelopmentLineProxy[] fDevelopmentLines;
		// Map of qualified parent team area paths (String) to a list (ArrayList)
		// of qualified child team area paths (String)
		HashMap fChildren = new HashMap();
		// List of root team area names (String)
		List fRoots = new ArrayList();
		// Map of root team area names (String) to the dev line they are
		// assigned to (IDevelopmentLineProxy)
		HashMap fRootDevLines = new HashMap();

		public Model(IDevelopmentLineProxy[] developmentLineProxies) {
			fDevelopmentLines = developmentLineProxies;
		}
		
		public void readFrom(IMemento memento) {
			IMemento[] rootMementos = memento.getChildren(TAG_TEAM_AREA);
			if (rootMementos != null) {
				for (int i = 0; i < rootMementos.length; i++) {
					IMemento rootMemento = rootMementos[i];
					if (rootMemento != null) {
						String developmentLine = normalize(rootMemento.getString(ATTR_DEVELOPMENT_LINE));
						String teamArea = normalize(rootMemento.getString(ATTR_NAME));
						createRoot(developmentLine, teamArea, rootMemento);
					}
				}
			}
		}
		
		private void createRoot(String devLineId, String teamAreaName, IMemento childMemento) {
			if (isNotNull(teamAreaName)) {
				fRoots.add(teamAreaName);
				if (devLineId != null) {
					IDevelopmentLineProxy devLine = findDevelopmentLine(devLineId);
					if (devLine != null) {
						fRootDevLines.put(teamAreaName, devLine);
					}
				}
				createChildren(teamAreaName, childMemento.getChildren(TAG_TEAM_AREA));
			}
		}

		private void createChild(String parent, String unqualifiedChild, IMemento childMemento) {
			if (isNotNull(parent) && isNotNull(unqualifiedChild)) {
				ArrayList children = (ArrayList) fChildren.get(parent);
				if (children == null) {
					children = new ArrayList();
					fChildren.put(parent, children);
				}
				String qualifiedChild = parent + SEPARATOR + unqualifiedChild;
				children.add(qualifiedChild);
				createChildren(qualifiedChild, childMemento.getChildren(TAG_TEAM_AREA));
			}
		}
		
		private void createChildren(String parent, IMemento[] childrenMemento) {
			if (childrenMemento != null) {
				for (int i = 0; i < childrenMemento.length; i++) {
					IMemento childMemento = childrenMemento[i];
					if (childMemento != null) {
						createChild(parent, normalize(childMemento.getString(ATTR_NAME)), childMemento);
					}
				}
			}
		}

		public boolean saveTo(IMemento memento) {
			for (Iterator iterator = fRoots.iterator(); iterator.hasNext();) {
				String rootTeamArea = (String) iterator.next();
				IMemento teamAreaMemento = memento.createChild(TAG_TEAM_AREA);
				teamAreaMemento.putString(ATTR_NAME, rootTeamArea);
				IDevelopmentLineProxy devLine = (IDevelopmentLineProxy) fRootDevLines.get(rootTeamArea);
				if (devLine != null) {
					teamAreaMemento.putString(ATTR_DEVELOPMENT_LINE, devLine.getId());
				}
				saveChildren(teamAreaMemento, rootTeamArea);
			}
			return fRoots.size() > 0;
		}

		private void saveChildren(IMemento parentMemento, String parent) {
			ArrayList children = (ArrayList) fChildren.get(parent);
			if (children != null && !children.isEmpty()) {
				for (Iterator e = children.iterator(); e.hasNext();) {
					String teamArea = (String) e.next();
					IMemento teamAreaMemento = parentMemento.createChild(TAG_TEAM_AREA);
					teamAreaMemento.putString(ATTR_NAME, getUnqualifiedName(teamArea));
					saveChildren(teamAreaMemento, teamArea);
				}
			}
		}
		
		public boolean remove(String qualifiedTeamArea) {
			removeChildren(qualifiedTeamArea);
			String parent = getQualifier(qualifiedTeamArea);
			if (parent == null) {
				fRoots.remove(qualifiedTeamArea);
				fRootDevLines.remove(qualifiedTeamArea);
				return true;
			}
			if (parent != null) {
				ArrayList children = (ArrayList) fChildren.get(parent);
				if (children != null) {
					boolean removed = children.remove(qualifiedTeamArea);
					if (children.isEmpty()) {
						fChildren.remove(parent);
					}
					return removed;
				}
			}
			return false;
		}

		private void removeChildren(String teamArea) {
			ArrayList children = (ArrayList) fChildren.remove(teamArea);
			if (children != null) {
				for (Iterator e = children.iterator(); e.hasNext();) {
					removeChildren((String) e.next());
				}
			}
		}
		
		public boolean exists(String parent, String unqualifiedTeamArea) {
			if (isNotNull(unqualifiedTeamArea)) {
				if (parent == null) {
					return fRoots.contains(unqualifiedTeamArea);
				}
				ArrayList children = (ArrayList) fChildren.get(parent);
				if (children != null) {
					return children.contains(parent + SEPARATOR + unqualifiedTeamArea);
				}
			}
			return false;
		}
		
		public String createRoot(String devLineId, String teamArea) {
			if (isNotNull(teamArea)) {
				if (fRoots.add(teamArea)) {
					IDevelopmentLineProxy line = findDevelopmentLine(devLineId);
					if (line != null) {
						fRootDevLines.put(teamArea, line);
					}
					return teamArea;
				}
			}
			return null;
		}
		
		public String createChild(String parent, String unqualifiedTeamArea) {
			if (isNotNull(parent) && isNotNull(unqualifiedTeamArea)) {
				ArrayList children = (ArrayList) fChildren.get(parent);
				if (children == null) {
					children = new ArrayList();
					fChildren.put(parent, children);
				}
				String qualifiedTeamArea = parent + SEPARATOR + unqualifiedTeamArea;
				children.add(qualifiedTeamArea);
				return qualifiedTeamArea;
			}
			return null;
		}
		
		public Object findParent(Object child) {
			if (child instanceof String) {
				return getQualifier((String) child);
			}
			return null;
		}
		
		private IDevelopmentLineProxy findDevelopmentLine(String devLineId) {
			if (devLineId != null) {
				for (int i = 0; i < fDevelopmentLines.length; i++) {
					IDevelopmentLineProxy proxy = fDevelopmentLines[i];
					if (proxy.getId().equals(devLineId)) {
						return proxy;
					}
				}
			}
			return null;
		}
	}
	
	private static String getUnqualifiedName(String qualifiedName) {
		int lastIndexOf = qualifiedName.lastIndexOf(Model.SEPARATOR);
		if (lastIndexOf > -1) {
			return qualifiedName.substring(lastIndexOf + 1);
		}
		return qualifiedName;
	}
	
	private static String getQualifier(String qualifiedName) {
		int lastIndexOf = qualifiedName.lastIndexOf(Model.SEPARATOR);
		if (lastIndexOf > -1) {
			return qualifiedName.substring(0, lastIndexOf);
		}
		return null;
	}
	
	private static boolean isNotNull(String string) {
		return string != null && string.trim().length() > 0;
	}
		
	private static String normalize(String string) {
		return string == null ? "" : string.trim(); //$NON-NLS-1$
	}
	
	
	private class TeamAreasContentProvider implements ITreeContentProvider {

		/*
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object element) {
			return getChildren(element);
		}

		/*
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {
		}

		/*
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}

		/*
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
		 */
		public Object[] getChildren(Object parentElement) {
			if (parentElement instanceof Model) {
				return ((Model) parentElement).fRoots.toArray();
			}
			ArrayList children = (ArrayList) fModel.fChildren.get(parentElement);
			if (children != null) {
				return children.toArray();
			}
			return new Object[0];
		}

		/*
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
		 */
		public Object getParent(Object element) {
			return fModel.findParent(element);
		}

		/*
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
		 */
		public boolean hasChildren(Object element) {
			ArrayList children = (ArrayList) fModel.fChildren.get(element);
			return children != null && !children.isEmpty();
		}
	}
	
	private class TeamAreasLabelProvider extends LabelProvider {
		
		private ResourceManager fResourceManager;
		
		
		public TeamAreasLabelProvider() {
			fResourceManager = new LocalResourceManager(JFaceResources.getResources());
		}
		
		/*
		 * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
		 */
		public String getText(Object element) {
			if (element instanceof String) {
				String name = super.getText(getUnqualifiedName((String) element));
				IDevelopmentLineProxy devLine = (IDevelopmentLineProxy) fModel.fRootDevLines.get(element);
				if (devLine != null) {
					name = NLS.bind(Messages.CreateTeamAreasAspectEditor_10, name, devLine.getName());
				}
				return normalize(name);
			}
			return super.getText(element);
		}
		
		/*
		 * @see org.eclipse.jface.viewers.LabelProvider#getImage(java.lang.Object)
		 */
		public Image getImage(Object element) {
			ImageDescriptor imageDesc = getImageDescriptor(element);
			if (imageDesc != null) {
				Image image = (Image) fResourceManager.find(imageDesc);
				if (image == null) {
					image = fResourceManager.createImageWithDefault(imageDesc);
				}
				return image;
			}
			return super.getImage(element);
		}
		
		private ImageDescriptor getImageDescriptor(Object element) {
			if (element instanceof String) {
				return ImagePool.TEAM_AREA_NOT_CUSTOMIZED;
			}
			return null;
		}
		
		/*
		 * @see org.eclipse.jface.viewers.BaseLabelProvider#dispose()
		 */
		public void dispose() {
			if (fResourceManager != null) {
				fResourceManager.dispose();
				fResourceManager = null;
			}
			super.dispose();
		}
	}
	
	private class DeleteTeamAreaAction extends Action {
		
		public DeleteTeamAreaAction() {
			setText(Messages.CreateTeamAreasAspectEditor_0);
			setToolTipText(Messages.CreateTeamAreasAspectEditor_1);
		}
		
		/*
		 * @see org.eclipse.jface.action.Action#run()
		 */
		public void run() {
			ISelection selection = fTeamAreasViewer.getSelection();
			if (selection instanceof IStructuredSelection) {
				IStructuredSelection ss = (IStructuredSelection) selection;
				for (Iterator e = ss.iterator(); e.hasNext();) {
					Object selected = e.next();
					if ( !(selected instanceof String)) {
						continue;
					}
					if (fModel.remove((String) selected)) {
						CreateTeamAreasAspectEditor.this.setDirty();
						fTeamAreasViewer.refresh();
					}
				}
			}
		}
	}
	
	private class CreateRootTeamAreaAction extends Action {
		
		public CreateRootTeamAreaAction() {
			setText(Messages.CreateTeamAreasAspectEditor_11);
			setToolTipText(Messages.CreateTeamAreasAspectEditor_12);
		}
		
		/*
		 * @see org.eclipse.jface.action.Action#run()
		 */
		public void run() {
			CreateRootTeamAreaDialog dialog= new CreateRootTeamAreaDialog(fTeamAreasViewer.getControl().getShell(), fModel.fDevelopmentLines, new IInputValidator() {
				public String isValid(String newText) {
					if (newText.length() == 0) {
						return ""; //$NON-NLS-1$
					}
					if (newText.indexOf(Model.SEPARATOR) > -1) {
						return Messages.CreateTeamAreasAspectEditor_7;
					}
					if (fModel.exists(null, newText)) {
						return Messages.CreateTeamAreasAspectEditor_8;
					}
					return null;
				}
			});
			if (dialog.open() == Window.OK) {
				String root = fModel.createRoot(dialog.getDevelopmentLine(), dialog.getName());
				if (root != null) {
					fTeamAreasViewer.refresh();
					fTeamAreasViewer.setSelection(new StructuredSelection(root), true);
					CreateTeamAreasAspectEditor.this.setDirty();
				}
			}
		}
	}
	
	private class CreateTeamAreaAction extends Action {
		
		public CreateTeamAreaAction() {
			setText(Messages.CreateTeamAreasAspectEditor_2);
			setToolTipText(Messages.CreateTeamAreasAspectEditor_3);
		}
		
		/*
		 * @see org.eclipse.jface.action.Action#run()
		 */
		public void run() {
			ISelection selection = fTeamAreasViewer.getSelection();
			if (selection instanceof IStructuredSelection) {
				IStructuredSelection ss = (IStructuredSelection) selection;
				
				String parent = null;
				Object selectedElement = ss.getFirstElement();
				if (selectedElement instanceof String) {
					parent = (String) selectedElement;
				}
				
				final String qualifier = parent;
				if (parent != null) {
					InputDialog dialog= new InputDialog(fTeamAreasViewer.getControl().getShell(), Messages.CreateTeamAreasAspectEditor_4, Messages.CreateTeamAreasAspectEditor_5, Messages.CreateTeamAreasAspectEditor_6, new IInputValidator() {
						public String isValid(String newText) {
							if (newText.length() == 0) {
								return ""; //$NON-NLS-1$
							}
							if (newText.indexOf(Model.SEPARATOR) > -1) {
								return Messages.CreateTeamAreasAspectEditor_7;
							}
							if (fModel.exists(qualifier, newText)) {
								return Messages.CreateTeamAreasAspectEditor_8;
							}
							return null;
						}
					});
					if (dialog.open() == Window.OK) {
						String child = fModel.createChild(parent, dialog.getValue());
						if (child != null) {
							fTeamAreasViewer.refresh(selectedElement);
							fTeamAreasViewer.setSelection(new StructuredSelection(child), true);
							CreateTeamAreasAspectEditor.this.setDirty();
						}
					}
				}
			}
		}
	}

	
	private Model fModel;
	private Button fCreateButton;
	private Button fCreateRootButton;
	private Button fDeleteButton;
	private TreeViewer fTeamAreasViewer;	

	
	public void createControl(Composite parent, FormToolkit toolkit) {
		GridLayoutFactory.fillDefaults().spacing(10, 8).numColumns(1).applyTo(parent);

		Label heading = toolkit.createLabel(parent, Messages.CreateTeamAreasAspectEditor_9);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(heading);
		heading.setFont(getBoldFont());
		
		Label description = toolkit.createLabel(parent, Messages.CreateTeamAreasAspectEditor_13, SWT.WRAP);
		int w = LayoutConstants.getSpacing().x * 30;
		GridDataFactory.fillDefaults().grab(true, false).hint(w, SWT.DEFAULT).applyTo(description);
		heading.setFont(getBoldFont());
		
		Composite body = toolkit.createComposite(parent);
		GridDataFactory.fillDefaults().grab(true, true).applyTo(body);
		GridLayoutFactory.fillDefaults().spacing(10, 8).numColumns(2).applyTo(body);
						
		Tree treeControl = toolkit.createTree(body, SWT.MULTI | SWT.BORDER);
		GridDataFactory.fillDefaults().grab(true, true).applyTo(treeControl);
		
		fTeamAreasViewer = new TreeViewer(treeControl);
		fTeamAreasViewer.setAutoExpandLevel(AbstractTreeViewer.ALL_LEVELS);
		fTeamAreasViewer.setComparator(new ViewerComparator());
		fTeamAreasViewer.setContentProvider(new TeamAreasContentProvider());
		fTeamAreasViewer.setLabelProvider(new TeamAreasLabelProvider());
		fTeamAreasViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				ISelection selection = event.getSelection();
				if (selection.isEmpty()) {
					fCreateButton.setEnabled(false);
					fDeleteButton.setEnabled(false);
				} else {
					IStructuredSelection ss = (IStructuredSelection) selection;
					if (ss.size() == 1) {
						fCreateButton.setEnabled(true);
					}
					fDeleteButton.setEnabled(true);
				}
			}
		});
		fTeamAreasViewer.setInput(fModel);

		final Action createTeamAreaAction = new CreateTeamAreaAction();
		final Action createRootTeamAreaAction = new CreateRootTeamAreaAction();
		final Action deleteTeamAreaAction = new DeleteTeamAreaAction();
		
		Composite toolbar = toolkit.createComposite(body);
		GridDataFactory.fillDefaults().align(SWT.RIGHT, SWT.TOP).grab(false, true).applyTo(toolbar);
		GridLayoutFactory.fillDefaults().spacing(5, 3).numColumns(1).applyTo(toolbar);
		
		fCreateButton = toolkit.createButton(toolbar, createTeamAreaAction.getText(), SWT.FLAT);
		fCreateButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false));
		fCreateButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				createTeamAreaAction.run();
			}
		});
		fCreateButton.setToolTipText(createTeamAreaAction.getToolTipText());
		fCreateButton.setEnabled(false);
		
		fCreateRootButton = toolkit.createButton(toolbar, createRootTeamAreaAction.getText(), SWT.FLAT);
		fCreateRootButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false));
		fCreateRootButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				createRootTeamAreaAction.run();
			}
		});
		fCreateRootButton.setToolTipText(createTeamAreaAction.getToolTipText());
		
		fDeleteButton = toolkit.createButton(toolbar, deleteTeamAreaAction.getText(), SWT.FLAT);
		fDeleteButton.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false));
		fDeleteButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				deleteTeamAreaAction.run();
			}
		});
		fDeleteButton.setToolTipText(deleteTeamAreaAction.getToolTipText());
		fDeleteButton.setEnabled(false);
	}
	
	private Font getBoldFont() {
		return JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT);
	}

	public void dispose() {
	}

	public void init(IProcessAspectEditorSite site, ProcessAspect inputAspect) {
		super.init(site, inputAspect);
		AbstractProcessAspect aspect = (AbstractProcessAspect) inputAspect;
		IIterationStructureProxy iterationStructureProxy = aspect.getIterationStructure();
		IDevelopmentLineProxy[] developmentLineProxies = iterationStructureProxy.getDevelopmentLines();
		fModel = new Model(developmentLineProxies);
	}

	public void restoreState(IMemento memento) {
		fModel.readFrom(memento);
	}

	public boolean saveState(IMemento memento) {
		return fModel.saveTo(memento);
	}		
}
