/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.process.internal.definitions.ide.ui.deliver.editors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.layout.GridDataFactory;

/**
 * Extends {@link InputDialog} to add a variables drop-down.
 */
public class InputDialogWithVars extends InputDialog {
	
	private String fTooltip;
	private String[][] fVars;

	public InputDialogWithVars(Shell parentShell, String dialogTitle, String dialogMessage, String initialValue, IInputValidator validator, String tooltip, String[][] vars) {
		super(parentShell, dialogTitle, dialogMessage, initialValue, validator);
		fTooltip = tooltip;
		fVars = vars;
	}

	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent);
		if (composite.getLayout() instanceof GridLayout) {
			GridLayout layout = (GridLayout) composite.getLayout();
			layout.numColumns++;
		}
		
		Control[] children = composite.getChildren();
		Text firstText = null;
		for (int i = 0; i < children.length; i++) {
			Control child = children[i];
			if (firstText == null && child instanceof Text) {
				firstText = (Text) child;
				if (fVars != null) {
					continue;
				}
			}
			if (child.getLayoutData() instanceof GridData) {
				GridData gridData = (GridData) child.getLayoutData();
				gridData.horizontalSpan++;
			}
		}
		
		if (firstText != null && fVars != null) {
			VarsButton varsButton = new VarsButton(composite, fTooltip, fVars);
			varsButton.setText(firstText);
			varsButton.getButton().moveBelow(firstText);
			GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.BEGINNING).applyTo(varsButton.getButton());
		}
		
		return composite;
	}
}
