/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2012. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.process.internal.definitions.ide.ui.deliver.editors;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.osgi.util.NLS;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.jface.window.Window;

import org.eclipse.ui.forms.widgets.FormToolkit;

import com.ibm.team.foundation.common.util.IMemento;
import com.ibm.team.jface.JazzResources;

import com.ibm.team.process.ide.ui.IProcessAspectEditorSite;
import com.ibm.team.process.ide.ui.ProcessAspect;

import com.ibm.team.process.internal.ide.ui.editors.form.AbstractProcessAspect;
import com.ibm.team.process.internal.ide.ui.editors.form.IDevelopmentLineProxy;
import com.ibm.team.process.internal.ide.ui.editors.form.IIterationStructureProxy;

/**
 * Aspect editor for the Setup Project follow-up action.
 */
public class SetupProjectAspectEditor extends CreateWorkItemsAspectEditor {

	/**
	 * A check-box with a bold, blue label whose text updates to have a trailing ':'
	 * when checked.
	 */
	static class SectionCheckbox {
		private String fText;
		private Button fButton;
		private Label fLabel;
		
		SectionCheckbox(Composite parent, FormToolkit toolkit, String text) {
			fText = text;
			// setting color on a button has no effect, so need to create an unlabeled check-box and a separate label instead
			Composite composite = toolkit.createComposite(parent);
			GridDataFactory.fillDefaults().grab(true, false).applyTo(composite);
			GridLayoutFactory.fillDefaults().numColumns(2).spacing(3, 0).applyTo(composite);
			fButton = toolkit.createButton(composite, "", SWT.CHECK); //$NON-NLS-1$
			fButton.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					updateLabel();
				}
			});
			fLabel = toolkit.createLabel(composite, text);
			GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).grab(true, false).applyTo(fLabel);
			fLabel.setFont(getBoldFont());
		}

		public boolean getSelection() {
			return fButton.getSelection();
		}

		public void addSelectionListener(SelectionListener listener) {
			fButton.addSelectionListener(listener);
		}

		public void setSelection(boolean selection) {
			fButton.setSelection(selection);
			updateLabel();
		}
		
		private void updateLabel() {
			fLabel.setText(fButton.getSelection() ? NLS.bind(Messages.SetupProjectAspectEditor_checkBoxSectionFmt, fText) : fText);
		}
	}


	static class Model {
		protected boolean createTeamArea = true;
		String teamAreaName = ""; //$NON-NLS-1$
		String developmentLineId = ""; //$NON-NLS-1$
		boolean createStream = true;
		String streamName = ""; //$NON-NLS-1$
		String streamDescription = ""; //$NON-NLS-1$
		List componentNames = new ArrayList();
		boolean createWorkspace = true;
		String workspaceName = ""; //$NON-NLS-1$
		String workspaceDescription = ""; //$NON-NLS-1$
		
		List categories = new ArrayList();
	}
	
	/**
	 * Model object for a work item category.
	 */
	static class Category {
		String path = ""; //$NON-NLS-1$
		String htmlDescription = ""; //$NON-NLS-1$
	}

	/**
	 * Form UI for a work item category
	 */
	class CategoryForm {

		private Category fCategory;
		private Composite fComposite;
		private Label fPathLabel;
		private Text fPathText;
		private Text fDescriptionText;

		CategoryForm(final Category category, Composite parent, FormToolkit toolkit, int index) {
			fCategory = category;
			fComposite = toolkit.createComposite(parent);
			GridDataFactory.fillDefaults().grab(true, false).applyTo(fComposite);
			GridLayoutFactory.fillDefaults().applyTo(fComposite);
			
			if (index != 0) {
				Label separator = toolkit.createLabel(fComposite, "", SWT.SEPARATOR | SWT.HORIZONTAL); //$NON-NLS-1$
				GridDataFactory.fillDefaults().grab(true, false).applyTo(separator);
			}

			Composite pathComposite = toolkit.createComposite(fComposite);
			GridDataFactory.fillDefaults().grab(true, false).applyTo(pathComposite);
			GridLayoutFactory.fillDefaults().numColumns(4).applyTo(pathComposite);
			
			fPathLabel = toolkit.createLabel(pathComposite, ""); //$NON-NLS-1$
			GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).applyTo(fPathLabel);
			
			fPathText = toolkit.createText(pathComposite, category.path, SWT.SINGLE | SWT.BORDER | SWT.SEARCH);
			fPathText.setMessage(Messages.SetupProjectAspectEditor_0);
			GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).grab(true, false).hint(150, -1).applyTo(fPathText);
			
			createVarsButton(pathComposite, fPathText, toolkit, CATEGORY_PATH_VARS);

			ToolBar toolbar = new ToolBar(pathComposite, SWT.NONE);
			toolkit.adapt(toolbar, true, false);
			ToolItem deleteItem = new ToolItem(toolbar, SWT.NONE);
			deleteItem.setImage(JazzResources.getImageWithDefault(fResourceManager, DELETE_IMAGE_DESCRIPTOR));
			deleteItem.setToolTipText(Messages.SetupProjectAspectEditor_1);
			GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).applyTo(toolbar);
			deleteItem.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					doDeleteCategory(category, CategoryForm.this);
				}
			});

			Composite descriptionComposite = toolkit.createComposite(fComposite);
			GridDataFactory.fillDefaults().grab(true, false).applyTo(descriptionComposite);
			GridLayoutFactory.fillDefaults().applyTo(descriptionComposite);
			
			Label descriptionLabel = toolkit.createLabel(descriptionComposite, Messages.AcceptTeamInvitationAspectEditor_1);
			GridDataFactory.fillDefaults().grab(true, false).align(SWT.FILL, SWT.CENTER).applyTo(descriptionLabel);
			
			fDescriptionText = toolkit.createText(descriptionComposite, category.htmlDescription, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL | SWT.BORDER);
			int h = fDescriptionText.getLineHeight() * 3;
			h = fDescriptionText.computeTrim(0, 0, 100, h).height;
			GridDataFactory.fillDefaults().grab(true, false).hint(150, h).applyTo(fDescriptionText);
			
			ModifyListener modifyListener = new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					if (e.widget == fPathText) {
						fCategory.path= fPathText.getText();
						setDirty();
					}
					else if (e.widget == fDescriptionText) {
						fCategory.htmlDescription = fDescriptionText.getText();
						setDirty();
					}
				}
			};
			fPathText.addModifyListener(modifyListener);
			fDescriptionText.addModifyListener(modifyListener);
			
			setIndex(index);
		}

		void setIndex(int index) {
			fPathLabel.setText(NLS.bind(Messages.SetupProjectAspectEditor_2, Integer.toString(index + 1)));
		}
		
		void dispose() {
			fComposite.dispose();
		}

		public void activate() {
			// hack: give the description field focus first, to have the managed form 
			// scroll and reveal more of the form if needed; without this,
			// only the path is revealed.
			fDescriptionText.setFocus();
			fPathText.setFocus();
			fPathText.selectAll();
		}
		
	}
	
	private static final String TAG_TEAM_AREA = "team-area"; //$NON-NLS-1$
	private static final String TAG_STREAM = "stream"; //$NON-NLS-1$
	private static final String TAG_COMPONENT = "component"; //$NON-NLS-1$
	private static final String TAG_WORKSPACE = "workspace"; //$NON-NLS-1$
	private static final String TAG_CATEGORY = "category"; //$NON-NLS-1$
	private static final String ATTR_NAME = "name"; //$NON-NLS-1$
	private static final String ATTR_DESCRIPTION = "description"; //$NON-NLS-1$
	private static final String ATTR_DEVELOPMENT_LINE = "timeline"; //$NON-NLS-1$
	private static final String ATTR_PATH = "path"; //$NON-NLS-1$
	
	private static final String[] VAR_STREAM_NAME = {Messages.SetupProjectAspectEditor_streamName, "${stream-name}"}; //$NON-NLS-1$

	private static final String[][] TEAM_AREA_NAME_VARS = { VAR_PROJECT_AREA_NAME };
	private static final String[][] STREAM_NAME_VARS = { VAR_PROJECT_AREA_NAME, VAR_TEAM_AREA_NAME };
	private static final String[][] WORKSPACE_NAME_VARS = { VAR_PROJECT_AREA_NAME, VAR_TEAM_AREA_NAME, VAR_STREAM_NAME };
	
	private Model fModel;
	private SectionCheckbox fCreateStreamButton;
	private Text fStreamNameText;
	private Text fStreamDescText;
	private ListViewer fComponentsViewer;
	private Button fAddComponentButton;
	private Button fRemoveComponentButton;
	private SectionCheckbox fCreateWorkspaceButton;
	private Text fWorkspaceNameText;
	private Text fWorkspaceDescText;
	private Composite fStreamDetailsComposite;
	
	private ModifyListener fModifyListener = new ModifyListener() {
		public void modifyText(ModifyEvent e) {
			if (e.widget == fTeamAreaNameText) {
				fModel.teamAreaName = fTeamAreaNameText.getText();
			}
			else if (e.widget == fStreamNameText) {
				fModel.streamName = fStreamNameText.getText();
			}
			else if (e.widget == fStreamDescText) {
				fModel.streamDescription = fStreamDescText.getText();
			}
			else if (e.widget == fWorkspaceNameText) {
				fModel.workspaceName = fWorkspaceNameText.getText();
			}
			else if (e.widget == fWorkspaceDescText) {
				fModel.workspaceDescription = fWorkspaceDescText.getText();
			}
			setDirty();
		}
	};
//	private Button fCreateTeamAreaButton;
	private Composite fTeamAreaDetailsComposite;
	private Text fTeamAreaNameText;
	private Composite fWorkspaceDetailsComposite;
	private IIterationStructureProxy fIterationStructure;
	private ComboViewer fDevelopmentLineComboViewer;
	
	private IPropertyChangeListener fIterationStructureChangeListener;
	private List fCategoryForms = new ArrayList();
	private Label fCategoryCountLabel;
	private Composite fCategoriesComposite;
	
	public SetupProjectAspectEditor() {
		super();
	}
	
	public void init(IProcessAspectEditorSite site, ProcessAspect inputAspect) {
		super.init(site, inputAspect);
		fModel = new Model();
		fIterationStructure = ((AbstractProcessAspect) getAspect()).getIterationStructure();
	}
	
	public void restoreState(IMemento memento) {
		IMemento teamAreaMemento = memento.getChild(TAG_TEAM_AREA);
		fModel.createTeamArea = /* teamAreaMemento != null */ true;  // this whole aspect is of not much use if a team area isn't created 
		fModel.teamAreaName = getAttrib(teamAreaMemento, ATTR_NAME, ""); //$NON-NLS-1$
		fModel.developmentLineId = getAttrib(teamAreaMemento, ATTR_DEVELOPMENT_LINE, ""); //$NON-NLS-1$

		IMemento streamMemento = memento.getChild(TAG_STREAM);
		fModel.createStream = streamMemento != null;
		fModel.streamName = getAttrib(streamMemento, ATTR_NAME, ""); //$NON-NLS-1$
		fModel.streamDescription = getAttrib(streamMemento, ATTR_DESCRIPTION, ""); //$NON-NLS-1$
		fModel.componentNames.clear();
		if (streamMemento != null) {
			IMemento[] componentMementos = streamMemento.getChildren(TAG_COMPONENT);
			for (int i = 0; i < componentMementos.length; i++) {
				fModel.componentNames.add(getAttrib(componentMementos[i], ATTR_NAME, "")); //$NON-NLS-1$
			}
		}

		IMemento workspaceMemento = memento.getChild(TAG_WORKSPACE);
		fModel.createWorkspace = workspaceMemento != null;
		fModel.workspaceName = getAttrib(workspaceMemento, ATTR_NAME, ""); //$NON-NLS-1$
		fModel.workspaceDescription = getAttrib(workspaceMemento, ATTR_DESCRIPTION, ""); //$NON-NLS-1$

		fModel.categories.clear();
		IMemento workitemsMemento = memento.getChild(TAG_WORKITEMS);
		if (workitemsMemento != null) {
			IMemento[] categoryMementos = workitemsMemento.getChildren(TAG_CATEGORY);
			for (int i = 0; i < categoryMementos.length; i++) {
				IMemento categoryMemento = categoryMementos[i];
				Category category = new Category();
				category.path = getAttrib(categoryMemento, ATTR_PATH, ""); //$NON-NLS-1$
				category.htmlDescription = getAttrib(categoryMemento, ATTR_HTML_DESCRIPTION, ""); //$NON-NLS-1$
				fModel.categories.add(category);
			}
		}
		
		super.restoreState(memento);
	}
	
	public boolean saveState(IMemento memento) {
		if (fModel.teamAreaName != null && fModel.teamAreaName.length() > 0) {
			IMemento teamAreaMemento = memento.createChild(TAG_TEAM_AREA);
			teamAreaMemento.putString(ATTR_NAME, fModel.teamAreaName);
			teamAreaMemento.putString(ATTR_DEVELOPMENT_LINE, fModel.developmentLineId);
		}
		
		if (fModel.createStream) {
			IMemento streamMemento = memento.createChild(TAG_STREAM);
			streamMemento.putString(ATTR_NAME, fModel.streamName);
			streamMemento.putString(ATTR_DESCRIPTION, convertLineSeparators(fModel.streamDescription));
			
			for (Iterator i = fModel.componentNames.iterator(); i.hasNext();) {
				String componentName = (String) i.next();
				streamMemento.createChild(TAG_COMPONENT).putString(ATTR_NAME, componentName);
			}
			
			if (fModel.createWorkspace) {
				IMemento workspaceMemento = memento.createChild(TAG_WORKSPACE);
				workspaceMemento.putString(ATTR_NAME, fModel.workspaceName);
				workspaceMemento.putString(ATTR_DESCRIPTION, convertLineSeparators(fModel.workspaceDescription));
			}
		}

		if (!fModel.categories.isEmpty()) {
			IMemento workitemsMemento = memento.createChild(TAG_WORKITEMS);
			for (Iterator i = fModel.categories.iterator(); i.hasNext();) {
				Category category = (Category) i.next();
				IMemento categoryMemento = workitemsMemento.createChild(TAG_CATEGORY);
				categoryMemento.putString(ATTR_PATH, category.path);
				categoryMemento.putString(ATTR_HTML_DESCRIPTION, convertLineSeparators(category.htmlDescription));
			}
		}
		return super.saveState(memento);
	}
	
	protected void createContents(Composite parent, FormToolkit toolkit) {
		Composite contentArea = createContentArea(parent, toolkit);
		createTeamAreaSection(contentArea, toolkit);
		createStreamAndWorkspaceSection(contentArea, toolkit);
		createCategoriesSection(contentArea, toolkit);
		createTemplatesHeader(contentArea, toolkit);
		hookModifyListeners();
	}

	private void createCategoriesSection(Composite parent, FormToolkit toolkit) {
		fCategoriesComposite = toolkit.createComposite(parent);
		GridDataFactory.fillDefaults().grab(true, true).applyTo(fCategoriesComposite);
		GridLayoutFactory.fillDefaults().spacing(0, 10).applyTo(fCategoriesComposite);
		createCategoriesHeader(fCategoriesComposite, toolkit);
	}

	private void createCategoriesHeader(Composite parent, FormToolkit toolkit) {
		Composite composite = toolkit.createComposite(parent);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(composite);
		GridLayoutFactory.fillDefaults().numColumns(2).applyTo(composite);
		
		fCategoryCountLabel = toolkit.createLabel(composite, ""); //$NON-NLS-1$
		fCategoryCountLabel.setFont(getBoldFont());
		GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).grab(true, true).applyTo(fCategoryCountLabel);
		
		Button newButton = toolkit.createButton(composite, Messages.SetupProjectAspectEditor_3, SWT.PUSH);
		newButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				doNewCategory();
			}
		});
	}


	private void updateVisibility(boolean reflowIfNeeded) {
		// note: don't use short-circuit evaluation here since we want all updateVisibility calls to run
		boolean reflowNeeded = updateVisibility(fModel.createTeamArea, fTeamAreaDetailsComposite);
		reflowNeeded |= updateVisibility(fModel.createWorkspace, fWorkspaceDetailsComposite);
		reflowNeeded |= updateVisibility(fModel.createStream, fStreamDetailsComposite);
		if (reflowNeeded && reflowIfNeeded) {
			getSite().reflow();
		}
	}

	private boolean updateVisibility(boolean makeVisible, Composite composite) {
		if (makeVisible != composite.getVisible()) {
			composite.setVisible(makeVisible);
			((GridData) composite.getLayoutData()).exclude = !makeVisible;
			return true;
		}
		return false;
	}

	protected void createTeamAreaSection(final Composite parent, FormToolkit toolkit) {
		Label sectionLabel = toolkit.createLabel(parent, Messages.SetupProjectAspectEditor_teamArea_section);
		sectionLabel.setFont(getBoldFont());
		GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).applyTo(sectionLabel);

//		fCreateTeamAreaButton = toolkit.createButton(parent, "Create a team area", SWT.CHECK);
//		GridDataFactory.fillDefaults().grab(true, false).span(3, 1).applyTo(fCreateTeamAreaButton);
//		fCreateTeamAreaButton.addSelectionListener(new SelectionAdapter() {
//			public void widgetSelected(SelectionEvent e) {
//				fModel.createTeamArea = fCreateTeamAreaButton.getSelection();
//				updateVisibility(true);
//			}
//		});

		fTeamAreaDetailsComposite = toolkit.createComposite(parent);
		GridDataFactory.fillDefaults().grab(true, false).span(2, 1).applyTo(fTeamAreaDetailsComposite);
		GridLayoutFactory.fillDefaults().extendedMargins(10, 0, 0, 5).spacing(10, 5).applyTo(fTeamAreaDetailsComposite);
		
		fTeamAreaNameText = createSingleLineText(fTeamAreaDetailsComposite, toolkit, Messages.SetupProjectAspectEditor_teamArea_label, Messages.SetupProjectAspectEditor_teamArea_message, TEAM_AREA_NAME_VARS);
		fDevelopmentLineComboViewer = createComboViewer(fTeamAreaDetailsComposite, toolkit, Messages.SetupProjectAspectEditor_devLine_label);
		fDevelopmentLineComboViewer.setContentProvider(new ArrayContentProvider());
		fDevelopmentLineComboViewer.setLabelProvider(new LabelProvider() {
			public String getText(Object element) {
				if (element instanceof IDevelopmentLineProxy) {
					IDevelopmentLineProxy line = (IDevelopmentLineProxy) element;
					return line.getName();
				}
				return super.getText(element);
			}
		});

		final Display display = parent.getDisplay();
		fIterationStructureChangeListener = new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				display.asyncExec(new Runnable() {
					public void run() {
						if (!parent.isDisposed()) {
							updateDevelopmentLines();
						}
					}
				});
			}
		};
		fIterationStructure.addPropertyChangeListener(fIterationStructureChangeListener);
		updateDevelopmentLines();
		
		fDevelopmentLineComboViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection sel = (IStructuredSelection) event.getSelection();
				String lineId = null;
				if (sel.size() == 1) {
					IDevelopmentLineProxy line = (IDevelopmentLineProxy) sel.getFirstElement();
					lineId = line.getId(); 
				}
				if (lineId == null) {
					lineId = ""; //$NON-NLS-1$
				}
				if (!lineId.equals(fModel.developmentLineId)) {
					fModel.developmentLineId = lineId;
					setDirty();
				}
			}
		});

	}

	private void updateDevelopmentLines() {
		IDevelopmentLineProxy[] developmentLines = fIterationStructure.getDevelopmentLines();
		fDevelopmentLineComboViewer.setInput(developmentLines);
		
		// update the selection
		for (int i = 0; i < developmentLines.length; i++) {
			IDevelopmentLineProxy line = developmentLines[i];
			if (line.getId().equals(fModel.developmentLineId)) {
				fDevelopmentLineComboViewer.setSelection(new StructuredSelection(line));
				return;
			}
		}
		fDevelopmentLineComboViewer.setSelection(StructuredSelection.EMPTY);
	}
	
	/*
	 * @see com.ibm.team.process.internal.definitions.ide.ui.deliver.editors.CreateWorkItemsAspectEditor#dispose()
	 */
	public void dispose() {
		super.dispose();
		if (fIterationStructureChangeListener != null) {
			fIterationStructure.removePropertyChangeListener(fIterationStructureChangeListener);
			fIterationStructureChangeListener = null;
		}
	}
	
	protected void createStreamAndWorkspaceSection(Composite parent, FormToolkit toolkit) {
		
		fCreateStreamButton = createSectionCheckbox(parent, toolkit, Messages.SetupProjectAspectEditor_stream_section);
		fCreateStreamButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				fModel.createStream = fCreateStreamButton.getSelection();
				updateVisibility(true);
			}
		});

		fStreamDetailsComposite = toolkit.createComposite(parent);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(fStreamDetailsComposite);
		GridLayoutFactory.fillDefaults().extendedMargins(0, 0, 0, 5).applyTo(fStreamDetailsComposite);
		
		Composite innerDetailsComposite = toolkit.createComposite(fStreamDetailsComposite);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(innerDetailsComposite);
		GridLayoutFactory.fillDefaults().extendedMargins(10, 0, 0, 5).applyTo(innerDetailsComposite);
		
		fStreamNameText = createSingleLineText(innerDetailsComposite, toolkit, Messages.SetupProjectAspectEditor_stream_label, Messages.SetupProjectAspectEditor_stream_message, STREAM_NAME_VARS);
		fStreamDescText = createMultiLineText(innerDetailsComposite, toolkit, Messages.SetupProjectAspectEditor_stream_desc_label, 3, STREAM_NAME_VARS);

		
		Label componentsLabel = toolkit.createLabel(innerDetailsComposite, Messages.SetupProjectAspectEditor_components_label);
		GridDataFactory.fillDefaults().applyTo(componentsLabel);
		
		Composite componentsComposite = toolkit.createComposite(innerDetailsComposite);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(componentsComposite);
		GridLayoutFactory.fillDefaults().numColumns(2).applyTo(componentsComposite);

		fComponentsViewer = new ListViewer(componentsComposite);
		fComponentsViewer.setContentProvider(new ArrayContentProvider());
		fComponentsViewer.setLabelProvider(new LabelProvider());
		fComponentsViewer.setSorter(new ViewerSorter());
		int h = fComponentsViewer.getList().getItemHeight() * 3;
		h = fComponentsViewer.getList().computeTrim(0, 0, 100, h).height;
		GridDataFactory.fillDefaults().grab(true, false).hint(100, h).applyTo(fComponentsViewer.getList());
		fComponentsViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				handleComponentSelection((IStructuredSelection) event.getSelection());
			}
		});
		
		Composite buttonsComposite = toolkit.createComposite(componentsComposite);
		GridDataFactory.fillDefaults().applyTo(buttonsComposite);
		GridLayoutFactory.fillDefaults().applyTo(buttonsComposite);
		
		fAddComponentButton = toolkit.createButton(buttonsComposite, Messages.SetupProjectAspectEditor_components_add, SWT.PUSH);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(fAddComponentButton);
		fAddComponentButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				doAddComponent();
			}
		});

		fRemoveComponentButton = toolkit.createButton(buttonsComposite, Messages.SetupProjectAspectEditor_components_remove, SWT.PUSH);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(fRemoveComponentButton);
		fRemoveComponentButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				doRemoveComponents();
			}
		});

		
		fCreateWorkspaceButton = createSectionCheckbox(fStreamDetailsComposite, toolkit, Messages.SetupProjectAspectEditor_workspace_section);
		fCreateWorkspaceButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				fModel.createWorkspace = fCreateWorkspaceButton.getSelection();
				updateVisibility(true);
			}
		});

		
		fWorkspaceDetailsComposite = toolkit.createComposite(fStreamDetailsComposite);
		GridDataFactory.fillDefaults().grab(true, false).span(2, 1).applyTo(fWorkspaceDetailsComposite);
		GridLayoutFactory.fillDefaults().extendedMargins(10, 0, 0, 0).applyTo(fWorkspaceDetailsComposite);

		fWorkspaceNameText = createSingleLineText(fWorkspaceDetailsComposite, toolkit, Messages.SetupProjectAspectEditor_workspace_label, Messages.SetupProjectAspectEditor_workspace_message, WORKSPACE_NAME_VARS);
		fWorkspaceDescText = createMultiLineText(fWorkspaceDetailsComposite, toolkit, Messages.SetupProjectAspectEditor_workspace_desc, 3, WORKSPACE_NAME_VARS);
		
		updateComponentButtonsEnablement(StructuredSelection.EMPTY);
	}

	protected void doAddComponent() {
		IInputValidator validator = new IInputValidator() {
			public String isValid(String newText) {
				if (newText.length() == 0) {
					return ""; //$NON-NLS-1$
				}
				return null;
			}
		};
		InputDialogWithVars inputDialog = new InputDialogWithVars(fComponentsViewer.getList().getShell(), Messages.SetupProjectAspectEditor_add_component_title, Messages.SetupProjectAspectEditor_add_component_message, Messages.SetupProjectAspectEditor_add_component_default, validator, VARS_TOOLTIP, WORKSPACE_NAME_VARS);
		if (inputDialog.open() == Window.OK) {
			String componentName = inputDialog.getValue();
			fModel.componentNames.add(componentName);
			fComponentsViewer.refresh();
			fComponentsViewer.setSelection(new StructuredSelection(componentName));
			setDirty();
		}
	}

	protected void doRemoveComponents() {
		IStructuredSelection selection = (IStructuredSelection) fComponentsViewer.getSelection();
		fModel.componentNames.removeAll(selection.toList());
		fComponentsViewer.refresh();
		setDirty();
	}

	private void handleComponentSelection(IStructuredSelection selection) {
		updateComponentButtonsEnablement(selection);
	}

	private void updateComponentButtonsEnablement(IStructuredSelection selection) {
		fRemoveComponentButton.setEnabled(!selection.isEmpty());
	}

	/**
	 * Creates a section check-box.  Note that an intermediate composite is created, so layout data
	 * should not be set on the result.
	 */
	private SectionCheckbox createSectionCheckbox(Composite parent, FormToolkit toolkit, String text) {
		return new SectionCheckbox(parent, toolkit, text);
	}
	
	private Text createMultiLineText(Composite parent, FormToolkit toolkit, String labelText, int numLines, String[][] vars) {
		Composite composite = toolkit.createComposite(parent);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(composite);
		GridLayoutFactory.fillDefaults().numColumns(2).applyTo(composite);
		
		Label label = toolkit.createLabel(composite, labelText);
		GridDataFactory.fillDefaults().grab(true, false).span(2, 1).applyTo(label);
		
		Text text = toolkit.createText(composite, "", SWT.MULTI | SWT.WRAP | SWT.V_SCROLL | SWT.BORDER); //$NON-NLS-1$
		int h = text.getLineHeight() * numLines;
		h = text.computeTrim(0, 0, 150, h).height;
		GridDataFactory.fillDefaults().grab(true, false).span(vars == null ? 2 : 1,1).hint(100, h).applyTo(text);
		
		if (vars != null) {
			createVarsButton(composite, text, toolkit, vars);
		}
		return text;
	}

	private Text createSingleLineText(Composite parent, FormToolkit toolkit, String labelText, String message, String[][] vars) {
		Composite composite = toolkit.createComposite(parent);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(composite);
		GridLayoutFactory.fillDefaults().numColumns(vars == null ? 2 : 3).applyTo(composite);
		
		Label label = toolkit.createLabel(composite, labelText);
		GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).applyTo(label);
		
		final Text text = toolkit.createText(composite, "", (message == null ? SWT.SINGLE : SWT.SEARCH) | SWT.BORDER); //$NON-NLS-1$
		GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).grab(true, false).hint(100, -1).applyTo(text);
		if (message != null) {
			text.setMessage(message);
		}
		
		if (vars != null) {
			createVarsButton(composite, text, toolkit, vars);
		}
		
		return text;
	}

	private ComboViewer createComboViewer(Composite parent, FormToolkit toolkit, String labelText) {
		Composite composite = toolkit.createComposite(parent);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(composite);
		GridLayoutFactory.fillDefaults().numColumns(2).applyTo(composite);
		
		Label label = toolkit.createLabel(composite, labelText);
		GridDataFactory.fillDefaults().align(SWT.FILL, SWT.CENTER).applyTo(label);
		
		ComboViewer comboViewer = new ComboViewer(composite);
		toolkit.adapt(comboViewer.getCombo(), true, false);
		GridDataFactory.fillDefaults().grab(true, false).hint(100, -1).applyTo(comboViewer.getCombo());

		return comboViewer;
	}

	private void hookModifyListeners() {
		fTeamAreaNameText.addModifyListener(fModifyListener);
		fStreamNameText.addModifyListener(fModifyListener);
		fStreamDescText.addModifyListener(fModifyListener);
		fWorkspaceNameText.addModifyListener(fModifyListener);
		fWorkspaceDescText.addModifyListener(fModifyListener);
	}

	private void unhookModifyListeners() {
		fTeamAreaNameText.removeModifyListener(fModifyListener);
		fStreamNameText.removeModifyListener(fModifyListener);
		fStreamDescText.removeModifyListener(fModifyListener);
		fWorkspaceNameText.removeModifyListener(fModifyListener);
		fWorkspaceDescText.removeModifyListener(fModifyListener);
	}

	protected void updateUI() {
//		fCreateTeamAreaButton.setSelection(fModel.createTeamArea);

		fCreateStreamButton.setSelection(fModel.createStream);
		fCreateWorkspaceButton.setSelection(fModel.createWorkspace);
		fComponentsViewer.setInput(fModel.componentNames);
		
		unhookModifyListeners();
		fTeamAreaNameText.setText(fModel.teamAreaName);
		fStreamDescText.setText(fModel.streamDescription);
		fStreamNameText.setText(fModel.streamName);
		fStreamDescText.setText(fModel.streamDescription);
		fWorkspaceNameText.setText(fModel.workspaceName);
		fWorkspaceDescText.setText(fModel.workspaceDescription);
		hookModifyListeners();
		
		updateVisibility(false);

		disposeCategoryForms();
		createCategoryForms();
		updateCategoryCount();
		
		super.updateUI();
	}

	private void updateCategoryCount() {
		fCategoryCountLabel.setText(NLS.bind(Messages.SetupProjectAspectEditor_4, Integer.toString(fModel.categories.size())));
	}

	private void createCategoryForms() {
		int index = 0;
		for (Iterator i = fModel.categories.iterator(); i.hasNext();) {
			Category category = (Category) i.next();
			fCategoryForms.add(new CategoryForm(category, fCategoriesComposite, fToolkit, index++));
		}
	}

	private void disposeCategoryForms() {
		for (Iterator i = fCategoryForms.iterator(); i.hasNext();) {
			CategoryForm form = (CategoryForm) i.next();
			form.dispose();
		}
		fCategoryForms.clear();
	}

	private void doNewCategory() {
		Category category = new Category();
		category.path = Messages.SetupProjectAspectEditor_5;
		category.htmlDescription = Messages.AcceptTeamInvitationAspectEditor_8;
		
		CategoryForm form = new CategoryForm(category, fCategoriesComposite, fToolkit, fModel.categories.size());
		fModel.categories.add(category);
		fCategoryForms.add(form);
		
		updateCategoryCount();
		setDirty();
		
		getContentArea().layout();
		getSite().reflow();

//		fScrolledForm.setOrigin(0, 65535); // scroll to bottom
		form.activate();
	}
	
	private void doDeleteCategory(Category category, CategoryForm form) {
		if (MessageDialog.openQuestion(getShell(), Messages.AcceptTeamInvitationAspectEditor_9, NLS.bind(Messages.SetupProjectAspectEditor_6, category.path))) {
			form.dispose();
			fModel.categories.remove(category);
			fCategoryForms.remove(form);
			
			updateCategoryCount();
			updateCategoryIndices();
			setDirty();
			
			getSite().reflow();
		}
	}

	private void updateCategoryIndices() {
		int index = 0;
		for (Iterator i = fCategoryForms.iterator(); i.hasNext();) {
			CategoryForm form = (CategoryForm) i.next();
			form.setIndex(index++);
		}
	}

}
