/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.process.internal.definitions.ide.ui.deliver.editors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Text;

import org.eclipse.jface.layout.GridDataFactory;

/**
 * An arrow button for inserting variables into a (usually adjacent) {@link Text} widget.
 */
public class VarsButton {

	private Button fButton;
	private Text fText;

	/**
	 * Creates the arrow button with the given variables.
	 * 
	 * @param parent the parent composite
	 * @param tooltip the tooltip to show on the button, or <code>null</code> for none
	 * @param vars the variables, an array of String[] where [0] is the variable name and [1] is the text to insert
	 */
	public VarsButton(Composite parent, String tooltip, String[][] vars) {
		// XXX: Hack: extend InputDialog and reach into its control structure to tweak the layout.
		
		final Button arrowButton = new Button(parent, SWT.ARROW | SWT.DOWN);
		fButton = arrowButton;
		
		if (tooltip != null) {
			arrowButton.setToolTipText(tooltip);
		}
		GridDataFactory.fillDefaults().align(SWT.BEGINNING , SWT.BEGINNING).applyTo(arrowButton);
		final Menu menu = new Menu(arrowButton);
		
		// dispose listener needed because menus are parented from the shell
		// and don't get disposed with the rest of the editor's controls otherwise
		arrowButton.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				menu.dispose();
			}
		});
		
		for (int i = 0; i < vars.length; ++i) {
			MenuItem menuItem = new MenuItem(menu, SWT.NONE);
			menuItem.setText(vars[i][0]);
			final String var = vars[i][1];
			menuItem.setData(vars[i][1]);
			menuItem.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					Text text = getText();
					if (text != null) {
						text.insert(var);
						text.setFocus();
					}
				}
			});
		}
		
		arrowButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Rectangle arrowBounds = arrowButton.getBounds();
				menu.setLocation(arrowButton.toDisplay(0, arrowBounds.height));
				menu.setVisible(true);
			}
		});
	}

	public Button getButton() {
		return fButton;
	}
	
	public Text getText() {
		return fText;
	}
	
	/**
	 * Set the text widget into which the chosen variables are inserted.
	 * 
	 * @param text the text widget
	 */
	public void setText(Text text) {
		fText = text;
	}

}
